# 
# Graphique H2 
# 
#----------------------------------------------------------------------
package provide Grh2::canvas 1.0

namespace eval Grh2 {
    variable dir;			# repertoire de base
    variable mainframe
    variable c
    variable scale 50
    variable xscroll
    variable yscroll
    variable toolbar
    variable toolbox
    variable tool 0
    variable displayList {}
    variable period 1000
    variable refresh 1

    namespace export mainframe c xscroll yscroll
}


# 
# Canvas 
#
proc Grh2::scrolledCanvas { c args } {

    # Creation canvas
    eval { canvas $c.canvas \
	       -xscrollcommand [list $c.xscroll set] \
	       -yscrollcommand [list $c.yscroll set] \
    } $args
    set width [ $c.canvas cget -width ]
    set height [ $c.canvas cget -height ]
    # Creation scroll bars 
    scrollbar $c.xscroll -orient horizontal -command [list $c.canvas xview]
    scrollbar $c.yscroll -orient vertical -command [list $c.canvas yview]
    # memorise dans les variables globales
    set ::Grh2::xscroll $c.xscroll
    set ::Grh2::yscroll $c.yscroll
    # placement 
    grid $c.canvas $c.yscroll -sticky news
    grid $c.xscroll -sticky ew
    grid rowconfigure $c 0 -weight 1
    grid columnconfigure $c 0 -weight 1

    bind $c.canvas <2> "$c.canvas scan mark %x %y"
    bind $c.canvas <B2-Motion> "$c.canvas scan dragto %x %y 1"

    return $c.canvas
}

#----------------------------------------------------------------------

# Recalcule les scrollBars

proc Grh2::updateScrollBars {} {
    # set height [winfo height $::Grh2::c]
    # set width [winfo width $::Grh2::c]

    set bbox [ $::Grh2::c bbox all ] 
    # set x1 [lindex $bbox 0]
    # set y1 [lindex $bbox 1]
    # set x2 [lindex $bbox 2]
    # set y2 [lindex $bbox 3]

    # definit le domaine de variation 
    $::Grh2::c configure -scrollregion $bbox
}

#----------------------------------------------------------------------

proc ::Grh2::grille {} {

    set imin [::Grh2::y_m_to_pixel [option get $::Grh2::c ymin YMin]]
    set jmin [::Grh2::x_m_to_pixel [option get $::Grh2::c xmin XMin]]
    set imax [::Grh2::y_m_to_pixel [option get $::Grh2::c ymax YMax]]
    set jmax [::Grh2::x_m_to_pixel [option get $::Grh2::c xmax XMax]]
    set pas [option get $::Grh2::c gridInterval GridInterval]
    set di [::Grh2::y_m_to_pixel $pas]
    set dj [::Grh2::x_m_to_pixel $pas]
    set color [ option get $::Grh2::c gridColor GridColor ]

    for { set i $imin } { $i >= $imax } { set i [expr $i + $di] } {
	$::Grh2::c create line $jmin $i $jmax $i -tag grille -fill $color
    }
    for { set j $jmin } { $j <= $jmax } { set j [expr $j + $dj] } {
     $::Grh2::c create line $j $imin $j $imax -tag grille -fill $color
    }

    $::Grh2::c create line 0 0 $dj 0 -arrow last -tag grille
    $::Grh2::c create line 0 0 0 $di -arrow last -tag grille
}
#----------------------------------------------------------------------

# Selectionne un objet graphique 

proc Grh2::select {} {
    $Grh2::c configure -cursor "left_ptr"
    bind $Grh2::c <Button-1> "Grh2::selectProc %x %y"
}

proc Grh2::selectProc {x y} {
    set cx [$Grh2::c canvasx $x]
    set cy [$Grh2::c canvasy $y]

    $Grh2::c create oval [expr $cx - 2] [expr $cy - 2] \
	[expr $cx + 2] [expr $cy + 2]
}


#----------------------------------------------------------------------

# Zoom en avant

proc Grh2::zoomin {} {
    $Grh2::c configure -cursor "@$::Grh2::dir/icons/zoomIn.xbm black"
    bind $Grh2::c <Button-1> "Grh2::zoominProc %x %y"
}

proc Grh2::zoominProc {x y} {
    
    set x [ $Grh2::c canvasx $x]
    set y [ $Grh2::c canvasy $y]
    set bbox [$::Grh2::c bbox all]
    set dx [expr ($x - [lindex $bbox 0])/([lindex $bbox 2] - [lindex $bbox 0])]
    set dy [expr ($y - [lindex $bbox 1])/([lindex $bbox 3] - [lindex $bbox 1])]

    # Modifie l'echelle 
    set ::Grh2::scale [expr $::Grh2::scale * 2]

    # ajuste le canvas 
    $Grh2::c scale all 0 0 2 2

    # ajuste les scrollbars
    Grh2::updateScrollBars
    $Grh2::c xview moveto $dx
    $Grh2::c xview scroll -5 units
    $Grh2::c yview moveto $dy
    $Grh2::c yview scroll -5 units
}

#----------------------------------------------------------------------

# Zoom en arriere

proc Grh2::zoomout {} {
    $Grh2::c configure -cursor "@$Grh2::dir/icons/zoomOut.xbm black"
    bind $Grh2::c <Button-1> "Grh2::zoomoutProc %x %y"
}

proc Grh2::zoomoutProc {x y} {

    set x [ $Grh2::c canvasx $x]
    set y [ $Grh2::c canvasy $y]
    set bbox [$::Grh2::c bbox all]
    set dx [expr ($x - [lindex $bbox 0])/([lindex $bbox 2] - [lindex $bbox 0])]
    set dy [expr ($y - [lindex $bbox 1])/([lindex $bbox 3] - [lindex $bbox 1])]

    # Modifie l'echelle
    set ::Grh2::scale [expr $::Grh2::scale / 2]

    # Ajuste le canvas
    $Grh2::c scale all 0 0 0.5 0.5
    # ajuste les scrollbars
    Grh2::updateScrollBars

    $Grh2::c xview moveto $dx
    $Grh2::c xview scroll -5 units
    $Grh2::c yview moveto $dy
    $Grh2::c yview scroll -5 units    
}

#----------------------------------------------------------------------

#
# Lecture du fichier de ressources initial
#
proc Grh2::getResources { appdefaults } {
    if [ file exists $appdefaults ] {
	if [catch { option readfile $appdefaults startup} err] {
	    puts stderr "error in $appdefaults: $err"
	}
    }
}

#----------------------------------------------------------------------

#
# Fin de l'appli
#
proc Grh2::exit {} {
    puts "\nBye...\n"
    ::exit 0
}

#----------------------------------------------------------------------

#
# Connexion
#
proc Grh2::connect {} {
    
    set rep [tk_entrydialog .connect "Connect to:"]

    if { $rep == "Cancel" } {
	return
    }
    ::connect $rep
    $Grh2::mainframe addindicator -text "Connected to $rep"
    return {} 
}
#----------------------------------------------------------------------

# Fonction appelee periodiquement pour la mise a jour de l'affichage

proc Grh2::timerFunc {} {
    foreach f $::Grh2::displayList {
	eval $f
    }
    if { $::Grh2::refresh } {
	after $::Grh2::period ::Grh2::timerFunc
    }
}

